import { defineStore } from "pinia";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";
import { useMemberStore } from "./member";
import type {
  CreateMemberEducationViewModel,
  MemberEducationViewModel,
  UpdateMemberEducationViewModel,
} from "@/viewmodels/admin/club/member/memberEducation.models";

export const useMemberEducationStore = defineStore("memberEducation", {
  state: () => {
    return {
      memberEducations: [] as Array<MemberEducationViewModel>,
      loading: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchMemberEducationsForMember() {
      const memberId = useMemberStore().activeMember;
      this.loading = "loading";
      http
        .get(`/admin/member/${memberId}/educations`)
        .then((result) => {
          this.memberEducations = result.data;
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    fetchMemberEducationById(id: number) {
      const memberId = useMemberStore().activeMember;
      return http.get(`/admin/member/${memberId}/education/${id}`);
    },
    async createMemberEducation(memberEducation: CreateMemberEducationViewModel): Promise<AxiosResponse<any, any>> {
      const memberId = useMemberStore().activeMember;
      const result = await http.post(`/admin/member/${memberId}/education`, {
        start: memberEducation.start,
        end: memberEducation.end,
        place: memberEducation.place,
        note: memberEducation.note,
        educationId: memberEducation.educationId,
      });
      this.fetchMemberEducationsForMember();
      return result;
    },
    async updateMemberEducation(memberEducation: UpdateMemberEducationViewModel): Promise<AxiosResponse<any, any>> {
      const memberId = useMemberStore().activeMember;
      const result = await http.patch(`/admin/member/${memberId}/education/${memberEducation.id}`, {
        start: memberEducation.start,
        end: memberEducation.end,
        place: memberEducation.place,
        note: memberEducation.note,
        educationId: memberEducation.educationId,
      });
      this.fetchMemberEducationsForMember();
      return result;
    },
    async deleteMemberEducation(memberEducation: number): Promise<AxiosResponse<any, any>> {
      const memberId = useMemberStore().activeMember;
      const result = await http.delete(`/admin/member/${memberId}/education/${memberEducation}`);
      this.fetchMemberEducationsForMember();
      return result;
    },
  },
});
