import { defineStore } from "pinia";
import type {
  CreateEducationViewModel,
  UpdateEducationViewModel,
  EducationViewModel,
} from "@/viewmodels/admin/configuration/education.models";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";

export const useEducationStore = defineStore("education", {
  state: () => {
    return {
      educations: [] as Array<EducationViewModel>,
      loading: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchEducations() {
      this.loading = "loading";
      http
        .get("/admin/education")
        .then((result) => {
          this.educations = result.data;
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    fetchEducationById(id: number): Promise<AxiosResponse<any, any>> {
      return http.get(`/admin/education/${id}`);
    },
    async createEducation(education: CreateEducationViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.post(`/admin/education`, {
        education: education.education,
        description: education.description,
      });
      this.fetchEducations();
      return result;
    },
    async updateActiveEducation(education: UpdateEducationViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.patch(`/admin/education/${education.id}`, {
        education: education.education,
        description: education.description,
      });
      this.fetchEducations();
      return result;
    },
    async deleteEducation(education: number): Promise<AxiosResponse<any, any>> {
      const result = await http.delete(`/admin/education/${education}`);
      this.fetchEducations();
      return result;
    },
  },
});
