import { defineStore } from "pinia";
import type {
  CreateSalutationViewModel,
  UpdateSalutationViewModel,
  SalutationViewModel,
} from "@/viewmodels/admin/configuration/salutation.models";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";

export const useSalutationStore = defineStore("salutation", {
  state: () => {
    return {
      salutations: [] as Array<SalutationViewModel>,
      loading: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchSalutations() {
      this.loading = "loading";
      http
        .get("/admin/salutation")
        .then((result) => {
          this.salutations = result.data;
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    fetchSalutationById(id: number): Promise<AxiosResponse<any, any>> {
      return http.get(`/admin/salutation/${id}`);
    },
    async createSalutation(salutation: CreateSalutationViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.post(`/admin/salutation`, {
        salutation: salutation.salutation,
      });
      this.fetchSalutations();
      return result;
    },
    async updateActiveSalutation(salutation: UpdateSalutationViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.patch(`/admin/salutation/${salutation.id}`, {
        salutation: salutation.salutation,
      });
      this.fetchSalutations();
      return result;
    },
    async deleteSalutation(salutation: number): Promise<AxiosResponse<any, any>> {
      const result = await http.delete(`/admin/salutation/${salutation}`);
      this.fetchSalutations();
      return result;
    },
  },
});
